@@ -326,3 +326,36 @@ static int s_sha256_hmac_test_invalid_state_fn(struct aws_allocator *allocator,
 }
 
 AWS_TEST_CASE(sha256_hmac_test_invalid_state, s_sha256_hmac_test_invalid_state_fn)
+
+static int s_sha256_hmac_test_extra_buffer_space_fn(struct aws_allocator *allocator, void *ctx) {
+    (void)ctx;
+
+    aws_cal_library_init(allocator);
+
+    uint8_t secret[] = {
+        0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d,
+        0x0e, 0x0f, 0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x19,
+    };
+    struct aws_byte_cursor secret_buf = aws_byte_cursor_from_array(secret, sizeof(secret));
+
+    struct aws_byte_cursor input = aws_byte_cursor_from_c_str("123456789012345678901234567890123456789012345"
+                                                              "67890123456789012345678901234567890");
+
+    struct aws_byte_buf digest_size_buf;
+    struct aws_byte_buf super_size_buf;
+
+    aws_byte_buf_init(&digest_size_buf, allocator, AWS_SHA256_HMAC_LEN);
+    aws_byte_buf_init(&super_size_buf, allocator, AWS_SHA256_HMAC_LEN + 100);
+
+    aws_sha256_hmac_compute(allocator, &secret_buf, &input, &digest_size_buf, 0);
+    aws_sha256_hmac_compute(allocator, &secret_buf, &input, &super_size_buf, 0);
+
+    ASSERT_TRUE(aws_byte_buf_eq(&digest_size_buf, &super_size_buf));
+    ASSERT_TRUE(super_size_buf.len == AWS_SHA256_HMAC_LEN);
+
+    aws_byte_buf_clean_up(&digest_size_buf);
+    aws_byte_buf_clean_up(&super_size_buf);
+
+    return AWS_OP_SUCCESS;
+}
+AWS_TEST_CASE(sha256_hmac_test_extra_buffer_space, s_sha256_hmac_test_extra_buffer_space_fn)
