@@ -269,23 +269,6 @@ bool aws_http_connection_is_server(const struct aws_http_connection *connection)
     return connection->server_data;
 }
 
-void aws_http_connection_update_window(struct aws_http_connection *connection, size_t increment_size) {
-    AWS_ASSERT(connection);
-    connection->vtable->update_window(connection, increment_size);
-}
-
-static int s_check_http2_connection(const struct aws_http_connection *http2_connection) {
-    if (http2_connection->http_version == AWS_HTTP_VERSION_2) {
-        return AWS_OP_SUCCESS;
-    } else {
-        AWS_LOGF_WARN(
-            AWS_LS_HTTP_CONNECTION,
-            "id=%p: HTTP/2 connection only function invoked on connection with other protocol, ignoring call.",
-            (void *)http2_connection);
-        return aws_raise_error(AWS_ERROR_INVALID_STATE);
-    }
-}
-
 int aws_http2_connection_change_settings(
     struct aws_http_connection *http2_connection,
     const struct aws_http2_setting *settings_array,
@@ -294,9 +277,7 @@ int aws_http2_connection_change_settings(
     void *user_data) {
     AWS_ASSERT(http2_connection);
     AWS_PRECONDITION(http2_connection->vtable);
-    if (s_check_http2_connection(http2_connection)) {
-        return AWS_OP_ERR;
-    }
+    AWS_FATAL_ASSERT(http2_connection->http_version == AWS_HTTP_VERSION_2);
     return http2_connection->vtable->change_settings(
         http2_connection, settings_array, num_settings, on_completed, user_data);
 }
@@ -308,9 +289,7 @@ int aws_http2_connection_ping(
     void *user_data) {
     AWS_ASSERT(http2_connection);
     AWS_PRECONDITION(http2_connection->vtable);
-    if (s_check_http2_connection(http2_connection)) {
-        return AWS_OP_ERR;
-    }
+    AWS_FATAL_ASSERT(http2_connection->http_version == AWS_HTTP_VERSION_2);
     return http2_connection->vtable->send_ping(http2_connection, optional_opaque_data, on_ack, user_data);
 }
 
@@ -321,9 +300,7 @@ int aws_http2_connection_send_goaway(
     const struct aws_byte_cursor *optional_debug_data) {
     AWS_ASSERT(http2_connection);
     AWS_PRECONDITION(http2_connection->vtable);
-    if (s_check_http2_connection(http2_connection)) {
-        return AWS_OP_ERR;
-    }
+    AWS_FATAL_ASSERT(http2_connection->http_version == AWS_HTTP_VERSION_2);
     return http2_connection->vtable->send_goaway(
         http2_connection, http2_error, allow_more_streams, optional_debug_data);
 }
@@ -336,9 +313,7 @@ int aws_http2_connection_get_sent_goaway(
     AWS_PRECONDITION(out_http2_error);
     AWS_PRECONDITION(out_last_stream_id);
     AWS_PRECONDITION(http2_connection->vtable);
-    if (s_check_http2_connection(http2_connection)) {
-        return AWS_OP_ERR;
-    }
+    AWS_FATAL_ASSERT(http2_connection->http_version == AWS_HTTP_VERSION_2);
     return http2_connection->vtable->get_sent_goaway(http2_connection, out_http2_error, out_last_stream_id);
 }
 
@@ -350,34 +325,33 @@ int aws_http2_connection_get_received_goaway(
     AWS_PRECONDITION(out_http2_error);
     AWS_PRECONDITION(out_last_stream_id);
     AWS_PRECONDITION(http2_connection->vtable);
-    if (s_check_http2_connection(http2_connection)) {
-        return AWS_OP_ERR;
-    }
+    AWS_FATAL_ASSERT(http2_connection->http_version == AWS_HTTP_VERSION_2);
     return http2_connection->vtable->get_received_goaway(http2_connection, out_http2_error, out_last_stream_id);
 }
 
-int aws_http2_connection_get_local_settings(
+void aws_http2_connection_get_local_settings(
     const struct aws_http_connection *http2_connection,
     struct aws_http2_setting out_settings[AWS_HTTP2_SETTINGS_COUNT]) {
     AWS_ASSERT(http2_connection);
     AWS_PRECONDITION(http2_connection->vtable);
-    if (s_check_http2_connection(http2_connection)) {
-        return AWS_OP_ERR;
-    }
+    AWS_FATAL_ASSERT(http2_connection->http_version == AWS_HTTP_VERSION_2);
     http2_connection->vtable->get_local_settings(http2_connection, out_settings);
-    return AWS_OP_SUCCESS;
 }
 
-int aws_http2_connection_get_remote_settings(
+void aws_http2_connection_get_remote_settings(
     const struct aws_http_connection *http2_connection,
     struct aws_http2_setting out_settings[AWS_HTTP2_SETTINGS_COUNT]) {
     AWS_ASSERT(http2_connection);
     AWS_PRECONDITION(http2_connection->vtable);
-    if (s_check_http2_connection(http2_connection)) {
-        return AWS_OP_ERR;
-    }
+    AWS_FATAL_ASSERT(http2_connection->http_version == AWS_HTTP_VERSION_2);
     http2_connection->vtable->get_remote_settings(http2_connection, out_settings);
-    return AWS_OP_SUCCESS;
+}
+
+void aws_http2_connection_update_window(struct aws_http_connection *http2_connection, uint32_t increment_size) {
+    AWS_ASSERT(http2_connection);
+    AWS_PRECONDITION(http2_connection->vtable);
+    AWS_FATAL_ASSERT(http2_connection->http_version == AWS_HTTP_VERSION_2);
+    http2_connection->vtable->update_window(http2_connection, increment_size);
 }
 
 struct aws_channel *aws_http_connection_get_channel(struct aws_http_connection *connection) {
