@@ -22,6 +22,7 @@ static int s_stream_get_sent_error_code(struct aws_http_stream *stream_base, uin
 
 static void s_stream_cross_thread_work_task(struct aws_channel_task *task, void *arg, enum aws_task_status status);
 static struct aws_h2err s_send_rst_and_close_stream(struct aws_h2_stream *stream, struct aws_h2err stream_error);
+static int s_stream_reset_stream_internal(struct aws_http_stream *stream_base, struct aws_h2err stream_error);
 
 struct aws_http_stream_vtable s_h2_stream_vtable = {
     .destroy = s_stream_destroy,
@@ -246,23 +247,44 @@ struct aws_h2_stream *aws_h2_stream_new_request(
 
     /* Init H2 specific stuff */
     stream->thread_data.state = AWS_H2_STREAM_STATE_IDLE;
-    stream->thread_data.outgoing_message = options->request;
+    enum aws_http_version message_version = aws_http_message_get_protocol_version(options->request);
+    switch (message_version) {
+        case AWS_HTTP_VERSION_1_1:
+            stream->thread_data.outgoing_message =
+                aws_http2_message_new_from_http1(options->request, stream->base.alloc);
+            if (!stream->thread_data.outgoing_message) {
+                AWS_H2_STREAM_LOG(ERROR, stream, "Stream failed to create the HTTP/2 message from HTTP/1.1 message");
+                goto error;
+            }
+            stream->backup_outgoing_message = options->request;
+            aws_http_message_acquire(stream->backup_outgoing_message);
+            break;
+        case AWS_HTTP_VERSION_2:
+            stream->thread_data.outgoing_message = options->request;
+            aws_http_message_acquire(stream->thread_data.outgoing_message);
+            break;
+        default:
+            /* Not supported */
+            aws_raise_error(AWS_ERROR_HTTP_UNSUPPORTED_PROTOCOL);
+            goto error;
+    }
 
     stream->sent_reset_error_code = -1;
     stream->received_reset_error_code = -1;
 
-    stream->synced_data.user_reset_error_code = AWS_HTTP2_ERR_COUNT;
+    stream->synced_data.reset_error.h2_code = AWS_HTTP2_ERR_COUNT;
     stream->synced_data.api_state = AWS_H2_STREAM_API_STATE_INIT;
     if (aws_mutex_init(&stream->synced_data.lock)) {
         AWS_H2_STREAM_LOGF(
             ERROR, stream, "Mutex init error %d (%s).", aws_last_error(), aws_error_name(aws_last_error()));
-        aws_mem_release(stream->base.alloc, stream);
-        return NULL;
+        goto error;
     }
-    aws_http_message_acquire(stream->thread_data.outgoing_message);
     aws_channel_task_init(
         &stream->cross_thread_work_task, s_stream_cross_thread_work_task, stream, "HTTP/2 stream cross-thread work");
     return stream;
+error:
+    s_stream_destroy(&stream->base);
+    return NULL;
 }
 
 static void s_stream_cross_thread_work_task(struct aws_channel_task *task, void *arg, enum aws_task_status status) {
@@ -286,7 +308,7 @@ static void s_stream_cross_thread_work_task(struct aws_channel_task *task, void
     bool ignore_window_update = (aws_h2_stream_get_state(stream) == AWS_H2_STREAM_STATE_HALF_CLOSED_REMOTE);
     bool reset_called;
     size_t window_update_size;
-    uint32_t user_reset_error_code;
+    struct aws_h2err reset_error;
 
     { /* BEGIN CRITICAL SECTION */
         s_lock_synced_data(stream);
@@ -296,7 +318,7 @@ static void s_stream_cross_thread_work_task(struct aws_channel_task *task, void
         window_update_size = stream->synced_data.window_update_size;
         stream->synced_data.window_update_size = 0;
         reset_called = stream->synced_data.reset_called;
-        user_reset_error_code = stream->synced_data.user_reset_error_code;
+        reset_error = stream->synced_data.reset_error;
 
         s_unlock_synced_data(stream);
     } /* END CRITICAL SECTION */
@@ -313,16 +335,7 @@ static void s_stream_cross_thread_work_task(struct aws_channel_task *task, void
     stream->thread_data.window_size_self += window_update_size;
 
     if (reset_called) {
-        struct aws_h2err h2err;
-        h2err.h2_code = user_reset_error_code;
-        if (stream->base.server_data && stream->thread_data.state == AWS_H2_STREAM_STATE_HALF_CLOSED_LOCAL) {
-            /* (RFC-7540 8.1) A server MAY request that the client abort transmission of a request without error by
-             * sending a RST_STREAM with an error code of NO_ERROR after sending a complete response */
-            h2err.aws_code = AWS_ERROR_SUCCESS;
-        } else {
-            h2err.aws_code = AWS_ERROR_HTTP_RST_STREAM_SENT;
-        }
-        struct aws_h2err returned_h2err = s_send_rst_and_close_stream(stream, h2err);
+        struct aws_h2err returned_h2err = s_send_rst_and_close_stream(stream, reset_error);
         if (aws_h2err_failed(returned_h2err)) {
             aws_h2_connection_shutdown_due_to_write_err(connection, returned_h2err.aws_code);
         }
@@ -343,6 +356,7 @@ static void s_stream_destroy(struct aws_http_stream *stream_base) {
     AWS_H2_STREAM_LOG(DEBUG, stream, "Destroying stream");
     aws_mutex_clean_up(&stream->synced_data.lock);
     aws_http_message_release(stream->thread_data.outgoing_message);
+    aws_http_message_release(stream->backup_outgoing_message);
 
     aws_mem_release(stream->base.alloc, stream);
 }
@@ -356,7 +370,8 @@ static void s_stream_update_window(struct aws_http_stream *stream_base, size_t i
     }
     if (!connection->base.manual_window_management) {
         /* auto-mode, manual update window is not supported */
-        AWS_H2_STREAM_LOG(WARN, stream, "Manual window management is off, update window operations are not supported.");
+        AWS_H2_STREAM_LOG(
+            DEBUG, stream, "Manual window management is off, update window operations are not supported.");
         return;
     }
 
@@ -399,18 +414,23 @@ static void s_stream_update_window(struct aws_http_stream *stream_base, size_t i
     if (err) {
         /* The increment_size is still not 100% safe, since we cannot control the incoming data frame. So just
          * ruled out the value that is obviously wrong values */
-        AWS_H2_STREAM_LOGF(
+        AWS_H2_STREAM_LOG(
             ERROR,
             stream,
-            "The increment size is too big for HTTP/2 protocol, max flow-control "
-            "window size is 2147483647. We got %zu, which will cause the flow-control window to exceed the maximum",
-            increment_size);
-        aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
-        return;
+            "The stream's flow-control window has been incremented beyond 2**31 -1, the max for HTTP/2. The stream "
+            "will close.");
+        aws_raise_error(AWS_ERROR_OVERFLOW_DETECTED);
+        struct aws_h2err stream_error = {
+            .aws_code = AWS_ERROR_OVERFLOW_DETECTED,
+            .h2_code = AWS_HTTP2_ERR_INTERNAL_ERROR,
+        };
+        /* Only when stream is not initialized reset will fail. So, we can assert it to be succeed. */
+        AWS_FATAL_ASSERT(s_stream_reset_stream_internal(stream_base, stream_error) == AWS_OP_SUCCESS);
     }
+    return;
 }
 
-static int s_stream_reset_stream(struct aws_http_stream *stream_base, uint32_t http2_error) {
+static int s_stream_reset_stream_internal(struct aws_http_stream *stream_base, struct aws_h2err stream_error) {
 
     struct aws_h2_stream *stream = AWS_CONTAINER_OF(stream_base, struct aws_h2_stream, base);
     struct aws_h2_connection *connection = s_get_h2_connection(stream);
@@ -426,11 +446,16 @@ static int s_stream_reset_stream(struct aws_http_stream *stream_base, uint32_t h
         if (!reset_called && !stream_is_init) {
             cross_thread_work_should_schedule = !stream->synced_data.is_cross_thread_work_task_scheduled;
             stream->synced_data.reset_called = true;
-            stream->synced_data.user_reset_error_code = http2_error;
+            stream->synced_data.reset_error = stream_error;
         }
         s_unlock_synced_data(stream);
     } /* END CRITICAL SECTION */
 
+    if (stream_is_init) {
+        AWS_H2_STREAM_LOG(
+            ERROR, stream, "Reset stream failed. Stream is in initialized state, please activate the stream first.");
+        return aws_raise_error(AWS_ERROR_INVALID_STATE);
+    }
     if (cross_thread_work_should_schedule) {
         AWS_H2_STREAM_LOG(TRACE, stream, "Scheduling stream cross-thread work task");
         /* increment the refcount of stream to keep it alive until the task runs */
@@ -438,13 +463,6 @@ static int s_stream_reset_stream(struct aws_http_stream *stream_base, uint32_t h
         aws_channel_schedule_task_now(connection->base.channel_slot->channel, &stream->cross_thread_work_task);
         return AWS_OP_SUCCESS;
     }
-
-    if (stream_is_init) {
-        AWS_H2_STREAM_LOG(
-            ERROR, stream, "Reset stream failed. Stream is in initialized state, please activate the stream first.");
-        return aws_raise_error(AWS_ERROR_INVALID_STATE);
-    }
-
     if (reset_called) {
         AWS_H2_STREAM_LOG(DEBUG, stream, "Reset stream ignored. Reset stream has been called already.");
     }
@@ -452,6 +470,21 @@ static int s_stream_reset_stream(struct aws_http_stream *stream_base, uint32_t h
     return AWS_OP_SUCCESS;
 }
 
+static int s_stream_reset_stream(struct aws_http_stream *stream_base, uint32_t http2_error) {
+    struct aws_h2err stream_error = {
+        .aws_code = AWS_ERROR_HTTP_RST_STREAM_SENT,
+        .h2_code = http2_error,
+    };
+
+    AWS_LOGF_TRACE(
+        AWS_LS_HTTP_STREAM,
+        "id=%p: User requested RST_STREAM with error code %s (0x%x)",
+        (void *)stream_base,
+        aws_http2_error_code_to_str(http2_error),
+        http2_error);
+    return s_stream_reset_stream_internal(stream_base, stream_error);
+}
+
 static int s_stream_get_received_error_code(struct aws_http_stream *stream_base, uint32_t *out_http2_error) {
     struct aws_h2_stream *stream = AWS_CONTAINER_OF(stream_base, struct aws_h2_stream, base);
     if (stream->received_reset_error_code == -1) {
@@ -537,13 +570,11 @@ int aws_h2_stream_on_activated(struct aws_h2_stream *stream, bool *out_has_outgo
 
     /* Create HEADERS frame */
     struct aws_http_message *msg = stream->thread_data.outgoing_message;
+    /* Should be ensured when the stream is created */
+    AWS_ASSERT(aws_http_message_get_protocol_version(msg) == AWS_HTTP_VERSION_2);
     bool has_body_stream = aws_http_message_get_body_stream(msg) != NULL;
-    struct aws_http_headers *h2_headers = aws_h2_create_headers_from_request(msg, stream->base.alloc);
-    if (!h2_headers) {
-        AWS_H2_STREAM_LOGF(
-            ERROR, stream, "Failed to create HTTP/2 style headers from request %s", aws_error_name(aws_last_error()));
-        goto error;
-    }
+    struct aws_http_headers *h2_headers = aws_http_message_get_headers(msg);
+
     struct aws_h2_frame *headers_frame = aws_h2_frame_new_headers(
         stream->base.alloc,
         stream->base.id,
@@ -552,8 +583,6 @@ int aws_h2_stream_on_activated(struct aws_h2_stream *stream, bool *out_has_outgo
         0 /* padding - not currently configurable via public API */,
         NULL /* priority - not currently configurable via public API */);
 
-    /* Release refcount of h2_headers here, let frame take the full ownership of it */
-    aws_http_headers_release(h2_headers);
     if (!headers_frame) {
         AWS_H2_STREAM_LOGF(ERROR, stream, "Failed to create HEADERS frame: %s", aws_error_name(aws_last_error()));
         goto error;
