@@ -5,11 +5,14 @@
 #include <aws/io/tls_channel_handler.h>
 
 #include <aws/common/clock.h>
+#include <aws/common/mutex.h>
 
 #include <aws/io/channel.h>
 #include <aws/io/event_loop.h>
 #include <aws/io/file_utils.h>
 #include <aws/io/logging.h>
+#include <aws/io/pkcs11.h>
+#include <aws/io/private/pkcs11_private.h>
 #include <aws/io/private/pki_utils.h>
 #include <aws/io/private/tls_channel_handler_shared.h>
 #include <aws/io/statistics.h>
@@ -44,6 +47,7 @@ struct s2n_handler {
     struct aws_channel_handler handler;
     struct aws_tls_channel_handler_shared shared_state;
     struct s2n_connection *connection;
+    struct s2n_ctx *s2n_ctx;
     struct aws_channel_slot *slot;
     struct aws_linked_list input_queue;
     struct aws_byte_buf protocol;
@@ -56,68 +60,108 @@ struct s2n_handler {
     aws_tls_on_error_fn *on_error;
     void *user_data;
     bool advertise_alpn_message;
-    bool negotiation_finished;
+    enum {
+        NEGOTIATION_ONGOING,
+        NEGOTIATION_FAILED,
+        NEGOTIATION_SUCCEEDED,
+    } state;
     struct s2n_delayed_shutdown_task delayed_shutdown_task;
+    struct aws_channel_task async_pkey_task;
 };
 
 struct s2n_ctx {
     struct aws_tls_ctx ctx;
     struct s2n_config *s2n_config;
+
+    /* Only used in special circumstances (ex: have cert but no key, because key is in PKCS#11) */
+    struct s2n_cert_chain_and_key *custom_cert_chain_and_key;
+
+    /* Use a single PKCS#11 session for all TLS connections on this s2n_ctx.
+     * We do this because PKCS#11 tokens may only support a
+     * limited number of sessions (PKCS11-UG-v2.40 section 2.6.7).
+     * If this one shared session turns out to be a severe bottleneck,
+     * we could look into other setups (ex: put session on its own thread,
+     * 1 session per event-loop, 1 session per connection, etc).
+     *
+     * The lock must be held while performing session operations.
+     * Otherwise, it would not be safe for multiple threads to share a
+     * session (PKCS11-UG-v2.40 section 2.6.7). The lock isn't needed for
+     * setup and teardown though, since we ensure nothing parallel is going
+     * on at these times */
+    struct {
+        struct aws_pkcs11_lib *lib;
+        struct aws_mutex session_lock;
+        CK_SESSION_HANDLE session_handle;
+        CK_OBJECT_HANDLE private_key_handle;
+        CK_KEY_TYPE private_key_type;
+    } pkcs11;
 };
 
+AWS_STATIC_STRING_FROM_LITERAL(s_debian_path, "/etc/ssl/certs");
+AWS_STATIC_STRING_FROM_LITERAL(s_rhel_path, "/etc/pki/tls/certs");
+AWS_STATIC_STRING_FROM_LITERAL(s_android_path, "/system/etc/security/cacerts");
+AWS_STATIC_STRING_FROM_LITERAL(s_free_bsd_path, "/usr/local/share/certs");
+AWS_STATIC_STRING_FROM_LITERAL(s_net_bsd_path, "/etc/openssl/certs");
+
 static const char *s_determine_default_pki_dir(void) {
     /* debian variants */
-    if (aws_path_exists("/etc/ssl/certs")) {
-        return "/etc/ssl/certs";
+    if (aws_path_exists(s_debian_path)) {
+        return aws_string_c_str(s_debian_path);
     }
 
     /* RHEL variants */
-    if (aws_path_exists("/etc/pki/tls/certs")) {
-        return "/etc/pki/tls/certs";
+    if (aws_path_exists(s_rhel_path)) {
+        return aws_string_c_str(s_rhel_path);
     }
 
     /* android */
-    if (aws_path_exists("/system/etc/security/cacerts")) {
-        return "/system/etc/security/cacerts";
+    if (aws_path_exists(s_android_path)) {
+        return aws_string_c_str(s_android_path);
     }
 
     /* Free BSD */
-    if (aws_path_exists("/usr/local/share/certs")) {
-        return "/usr/local/share/certs";
+    if (aws_path_exists(s_free_bsd_path)) {
+        return aws_string_c_str(s_free_bsd_path);
     }
 
     /* Net BSD */
-    if (aws_path_exists("/etc/openssl/certs")) {
-        return "/etc/openssl/certs";
+    if (aws_path_exists(s_net_bsd_path)) {
+        return aws_string_c_str(s_net_bsd_path);
     }
 
     return NULL;
 }
 
+AWS_STATIC_STRING_FROM_LITERAL(s_debian_ca_file_path, "/etc/ssl/certs/ca-certificates.crt");
+AWS_STATIC_STRING_FROM_LITERAL(s_old_rhel_ca_file_path, "/etc/pki/tls/certs/ca-bundle.crt");
+AWS_STATIC_STRING_FROM_LITERAL(s_open_suse_ca_file_path, "/etc/ssl/ca-bundle.pem");
+AWS_STATIC_STRING_FROM_LITERAL(s_open_elec_ca_file_path, "/etc/pki/tls/cacert.pem");
+AWS_STATIC_STRING_FROM_LITERAL(s_modern_rhel_ca_file_path, "/etc/pki/ca-trust/extracted/pem/tls-ca-bundle.pem");
+
 static const char *s_determine_default_pki_ca_file(void) {
     /* debian variants */
-    if (aws_path_exists("/etc/ssl/certs/ca-certificates.crt")) {
-        return "/etc/ssl/certs/ca-certificates.crt";
+    if (aws_path_exists(s_debian_ca_file_path)) {
+        return aws_string_c_str(s_debian_ca_file_path);
     }
 
     /* Old RHEL variants */
-    if (aws_path_exists("/etc/pki/tls/certs/ca-bundle.crt")) {
-        return "/etc/pki/tls/certs/ca-bundle.crt";
+    if (aws_path_exists(s_old_rhel_ca_file_path)) {
+        return aws_string_c_str(s_old_rhel_ca_file_path);
     }
 
     /* Open SUSE */
-    if (aws_path_exists("/etc/ssl/ca-bundle.pem")) {
-        return "/etc/ssl/ca-bundle.pem";
+    if (aws_path_exists(s_open_suse_ca_file_path)) {
+        return aws_string_c_str(s_open_suse_ca_file_path);
     }
 
     /* Open ELEC */
-    if (aws_path_exists("/etc/pki/tls/cacert.pem")) {
-        return "/etc/pki/tls/cacert.pem";
+    if (aws_path_exists(s_open_elec_ca_file_path)) {
+        return aws_string_c_str(s_open_elec_ca_file_path);
     }
 
     /* Modern RHEL variants */
-    if (aws_path_exists("/etc/pki/ca-trust/extracted/pem/tls-ca-bundle.pem")) {
-        return "/etc/pki/ca-trust/extracted/pem/tls-ca-bundle.pem";
+    if (aws_path_exists(s_modern_rhel_ca_file_path)) {
+        return aws_string_c_str(s_modern_rhel_ca_file_path);
     }
 
     return NULL;
@@ -159,6 +203,7 @@ bool aws_tls_is_cipher_pref_supported(enum aws_tls_cipher_pref cipher_pref) {
         case AWS_IO_TLS_CIPHER_PREF_KMS_PQ_TLSv1_0_2020_02:
         case AWS_IO_TLS_CIPHER_PREF_KMS_PQ_SIKE_TLSv1_0_2020_02:
         case AWS_IO_TLS_CIPHER_PREF_KMS_PQ_TLSv1_0_2020_07:
+        case AWS_IO_TLS_CIPHER_PREF_PQ_TLSv1_0_2021_05:
             return true;
 #endif
 
@@ -270,7 +315,12 @@ static void s_s2n_handler_destroy(struct aws_channel_handler *handler) {
     if (handler) {
         struct s2n_handler *s2n_handler = (struct s2n_handler *)handler->impl;
         aws_tls_channel_handler_shared_clean_up(&s2n_handler->shared_state);
-        s2n_connection_free(s2n_handler->connection);
+        if (s2n_handler->connection) {
+            s2n_connection_free(s2n_handler->connection);
+        }
+        if (s2n_handler->s2n_ctx) {
+            aws_tls_ctx_release(&s2n_handler->s2n_ctx->ctx);
+        }
         aws_mem_release(handler->alloc, (void *)s2n_handler);
     }
 }
@@ -293,6 +343,8 @@ static void s_on_negotiation_result(
 static int s_drive_negotiation(struct aws_channel_handler *handler) {
     struct s2n_handler *s2n_handler = (struct s2n_handler *)handler->impl;
 
+    AWS_ASSERT(s2n_handler->state == NEGOTIATION_ONGOING);
+
     aws_on_drive_tls_negotiation(&s2n_handler->shared_state);
 
     s2n_blocked_status blocked = S2N_NOT_BLOCKED;
@@ -301,7 +353,7 @@ static int s_drive_negotiation(struct aws_channel_handler *handler) {
 
         int s2n_error = s2n_errno;
         if (negotiation_code == S2N_ERR_T_OK) {
-            s2n_handler->negotiation_finished = true;
+            s2n_handler->state = NEGOTIATION_SUCCEEDED;
 
             const char *protocol = s2n_get_application_protocol(s2n_handler->connection);
             if (protocol) {
@@ -356,7 +408,7 @@ static int s_drive_negotiation(struct aws_channel_handler *handler) {
 
             const char *err_str = s2n_strerror_debug(s2n_error, NULL);
             (void)err_str;
-            s2n_handler->negotiation_finished = false;
+            s2n_handler->state = NEGOTIATION_FAILED;
 
             aws_raise_error(AWS_IO_TLS_ERROR_NEGOTIATION_FAILURE);
 
@@ -376,7 +428,10 @@ static void s_negotiation_task(struct aws_channel_task *task, void *arg, aws_tas
 
     if (status == AWS_TASK_STATUS_RUN_READY) {
         struct aws_channel_handler *handler = arg;
-        s_drive_negotiation(handler);
+        struct s2n_handler *s2n_handler = (struct s2n_handler *)handler->impl;
+        if (s2n_handler->state == NEGOTIATION_ONGOING) {
+            s_drive_negotiation(handler);
+        }
     }
 }
 
@@ -385,7 +440,10 @@ int aws_tls_client_handler_start_negotiation(struct aws_channel_handler *handler
 
     AWS_LOGF_TRACE(AWS_LS_IO_TLS, "id=%p: Kicking off TLS negotiation.", (void *)handler)
     if (aws_channel_thread_is_callers_thread(s2n_handler->slot->channel)) {
-        return s_drive_negotiation(handler);
+        if (s2n_handler->state == NEGOTIATION_ONGOING) {
+            s_drive_negotiation(handler);
+        }
+        return AWS_OP_SUCCESS;
     }
 
     aws_channel_task_init(
@@ -402,10 +460,14 @@ static int s_s2n_handler_process_read_message(
 
     struct s2n_handler *s2n_handler = handler->impl;
 
+    if (AWS_UNLIKELY(s2n_handler->state == NEGOTIATION_FAILED)) {
+        return aws_raise_error(AWS_IO_TLS_ERROR_NEGOTIATION_FAILURE);
+    }
+
     if (message) {
         aws_linked_list_push_back(&s2n_handler->input_queue, &message->queueing_handle);
 
-        if (!s2n_handler->negotiation_finished) {
+        if (s2n_handler->state == NEGOTIATION_ONGOING) {
             size_t message_len = message->message_data.len;
             if (!s_drive_negotiation(handler)) {
                 aws_channel_slot_increment_read_window(slot, message_len);
@@ -494,7 +556,7 @@ static int s_s2n_handler_process_write_message(
     (void)slot;
     struct s2n_handler *s2n_handler = (struct s2n_handler *)handler->impl;
 
-    if (AWS_UNLIKELY(!s2n_handler->negotiation_finished)) {
+    if (AWS_UNLIKELY(s2n_handler->state != NEGOTIATION_SUCCEEDED)) {
         return aws_raise_error(AWS_IO_TLS_ERROR_NOT_NEGOTIATED);
     }
 
@@ -537,6 +599,231 @@ static void s_delayed_shutdown_task_fn(struct aws_channel_task *channel_task, vo
         false);
 }
 
+static enum aws_tls_signature_algorithm s_s2n_to_aws_signature_algorithm(s2n_tls_signature_algorithm s2n_alg) {
+    switch (s2n_alg) {
+        case S2N_TLS_SIGNATURE_RSA:
+            return AWS_TLS_SIGNATURE_RSA;
+        default:
+            return AWS_TLS_SIGNATURE_UNKNOWN;
+    }
+}
+
+static enum aws_tls_hash_algorithm s_s2n_to_aws_hash_algorithm(s2n_tls_hash_algorithm s2n_alg) {
+    switch (s2n_alg) {
+        case (S2N_TLS_HASH_SHA1):
+            return AWS_TLS_HASH_SHA1;
+        case (S2N_TLS_HASH_SHA224):
+            return AWS_TLS_HASH_SHA224;
+        case (S2N_TLS_HASH_SHA256):
+            return AWS_TLS_HASH_SHA256;
+        case (S2N_TLS_HASH_SHA384):
+            return AWS_TLS_HASH_SHA384;
+        case (S2N_TLS_HASH_SHA512):
+            return AWS_TLS_HASH_SHA512;
+        default:
+            return AWS_TLS_HASH_UNKNOWN;
+    }
+}
+
+/* This task performs the PKCS#11 private key operations.
+ * This task is scheduled because the s2n async private key operation is not allowed to complete synchronously */
+static void s_s2n_pkcs11_async_pkey_task(
+    struct aws_channel_task *channel_task,
+    void *arg,
+    enum aws_task_status status) {
+
+    struct s2n_handler *s2n_handler = AWS_CONTAINER_OF(channel_task, struct s2n_handler, async_pkey_task);
+    struct aws_channel_handler *handler = &s2n_handler->handler;
+    struct s2n_async_pkey_op *op = arg;
+    bool success = false;
+
+    uint8_t *input_data = NULL;     /* allocated later */
+    struct aws_byte_buf output_buf; /* initialized later */
+    AWS_ZERO_STRUCT(output_buf);
+
+    /* if things started failing since this task was scheduled, just clean up and bail out */
+    if (status != AWS_TASK_STATUS_RUN_READY || s2n_handler->state != NEGOTIATION_ONGOING) {
+        goto clean_up;
+    }
+
+    AWS_LOGF_TRACE(AWS_LS_IO_TLS, "id=%p: Running PKCS#11 async pkey task", (void *)handler);
+
+    /* We check all s2n_async_pkey_op functions for success,
+     * but they shouldn't fail if they're called correctly.
+     * Even if the output is bad, the failure will happen later in s2n_negotiate() */
+
+    uint32_t input_size = 0;
+    if (s2n_async_pkey_op_get_input_size(op, &input_size)) {
+        AWS_LOGF_ERROR(AWS_LS_IO_TLS, "id=%p: Failed querying s2n async pkey op size", (void *)handler);
+        aws_raise_error(AWS_ERROR_INVALID_STATE);
+        goto error;
+    }
+
+    input_data = aws_mem_acquire(handler->alloc, input_size);
+    if (s2n_async_pkey_op_get_input(op, input_data, input_size)) {
+        AWS_LOGF_ERROR(AWS_LS_IO_TLS, "id=%p: Failed querying s2n async pkey input", (void *)handler);
+        aws_raise_error(AWS_ERROR_INVALID_STATE);
+        goto error;
+    }
+    struct aws_byte_cursor input_cursor = aws_byte_cursor_from_array(input_data, input_size);
+
+    s2n_async_pkey_op_type op_type = 0;
+    if (s2n_async_pkey_op_get_op_type(op, &op_type)) {
+        AWS_LOGF_ERROR(AWS_LS_IO_TLS, "id=%p: Failed querying s2n async pkey op type", (void *)handler);
+        aws_raise_error(AWS_ERROR_INVALID_STATE);
+        goto error;
+    }
+
+    /* Gather additional information if this is a SIGN operation */
+    enum aws_tls_signature_algorithm aws_sign_alg = 0;
+    enum aws_tls_hash_algorithm aws_digest_alg = 0;
+    if (op_type == S2N_ASYNC_SIGN) {
+        s2n_tls_signature_algorithm s2n_sign_alg = 0;
+        if (s2n_connection_get_selected_client_cert_signature_algorithm(s2n_handler->connection, &s2n_sign_alg)) {
+            AWS_LOGF_ERROR(AWS_LS_IO_TLS, "id=%p: Failed getting s2n client cert signature algorithm", (void *)handler);
+            aws_raise_error(AWS_ERROR_INVALID_STATE);
+            goto error;
+        }
+
+        aws_sign_alg = s_s2n_to_aws_signature_algorithm(s2n_sign_alg);
+        if (aws_sign_alg == AWS_TLS_SIGNATURE_UNKNOWN) {
+            AWS_LOGF_ERROR(
+                AWS_LS_IO_TLS,
+                "id=%p: Cannot sign with s2n_tls_signature_algorithm=%d. Algorithm currently unsupported",
+                (void *)handler,
+                s2n_sign_alg);
+            aws_raise_error(AWS_IO_TLS_SIGNATURE_ALGORITHM_UNSUPPORTED);
+            goto error;
+        }
+
+        s2n_tls_hash_algorithm s2n_digest_alg = 0;
+        if (s2n_connection_get_selected_client_cert_digest_algorithm(s2n_handler->connection, &s2n_digest_alg)) {
+            AWS_LOGF_ERROR(AWS_LS_IO_TLS, "id=%p: Failed getting s2n client cert digest algorithm", (void *)handler);
+            aws_raise_error(AWS_ERROR_INVALID_STATE);
+            goto error;
+        }
+
+        aws_digest_alg = s_s2n_to_aws_hash_algorithm(s2n_digest_alg);
+        if (aws_digest_alg == AWS_TLS_HASH_UNKNOWN) {
+            AWS_LOGF_ERROR(
+                AWS_LS_IO_TLS,
+                "id=%p: Cannot sign digest created with s2n_tls_hash_algorithm=%d. Algorithm currently unsupported",
+                (void *)handler,
+                s2n_digest_alg);
+            aws_raise_error(AWS_IO_TLS_DIGEST_ALGORITHM_UNSUPPORTED);
+            goto error;
+        }
+    }
+
+    /*********** BEGIN CRITICAL SECTION ***********/
+    aws_mutex_lock(&s2n_handler->s2n_ctx->pkcs11.session_lock);
+    bool success_while_locked = false;
+
+    switch (op_type) {
+        case S2N_ASYNC_DECRYPT:
+            if (aws_pkcs11_lib_decrypt(
+                    s2n_handler->s2n_ctx->pkcs11.lib,
+                    s2n_handler->s2n_ctx->pkcs11.session_handle,
+                    s2n_handler->s2n_ctx->pkcs11.private_key_handle,
+                    s2n_handler->s2n_ctx->pkcs11.private_key_type,
+                    input_cursor,
+                    handler->alloc,
+                    &output_buf)) {
+
+                AWS_LOGF_ERROR(
+                    AWS_LS_IO_TLS,
+                    "id=%p: PKCS#11 decrypt failed, error %s",
+                    (void *)handler,
+                    aws_error_name(aws_last_error()));
+                goto unlock;
+            }
+            break;
+
+        case S2N_ASYNC_SIGN:
+            if (aws_pkcs11_lib_sign(
+                    s2n_handler->s2n_ctx->pkcs11.lib,
+                    s2n_handler->s2n_ctx->pkcs11.session_handle,
+                    s2n_handler->s2n_ctx->pkcs11.private_key_handle,
+                    s2n_handler->s2n_ctx->pkcs11.private_key_type,
+                    input_cursor,
+                    handler->alloc,
+                    aws_digest_alg,
+                    aws_sign_alg,
+                    &output_buf)) {
+
+                AWS_LOGF_ERROR(
+                    AWS_LS_IO_TLS,
+                    "id=%p: PKCS#11 sign failed, error %s",
+                    (void *)handler,
+                    aws_error_name(aws_last_error()));
+                goto unlock;
+            }
+            break;
+
+        default:
+            AWS_LOGF_ERROR(AWS_LS_IO_TLS, "id=%p: Unknown s2n_async_pkey_op_type:%d", (void *)handler, (int)op_type);
+            aws_raise_error(AWS_ERROR_INVALID_STATE);
+            goto unlock;
+    }
+
+    success_while_locked = true;
+unlock:
+    aws_mutex_unlock(&s2n_handler->s2n_ctx->pkcs11.session_lock);
+    /*********** END CRITICAL SECTION ***********/
+
+    if (!success_while_locked) {
+        goto error;
+    }
+
+    AWS_LOGF_TRACE(
+        AWS_LS_IO_TLS, "id=%p: PKCS#11 operation complete. output-size:%zu", (void *)handler, output_buf.len);
+
+    if (s2n_async_pkey_op_set_output(op, output_buf.buffer, output_buf.len)) {
+        AWS_LOGF_ERROR(AWS_LS_IO_TLS, "id=%p: Failed setting output on s2n async pkey op", (void *)handler);
+        aws_raise_error(AWS_ERROR_INVALID_STATE);
+        goto error;
+    }
+
+    if (s2n_async_pkey_op_apply(op, s2n_handler->connection)) {
+        AWS_LOGF_ERROR(AWS_LS_IO_TLS, "id=%p: Failed applying s2n async pkey op", (void *)handler);
+        aws_raise_error(AWS_ERROR_INVALID_STATE);
+        goto error;
+    }
+
+    /* Success! */
+    success = true;
+    goto clean_up;
+
+error:
+    aws_channel_shutdown(s2n_handler->slot->channel, aws_last_error());
+
+clean_up:
+    s2n_async_pkey_op_free(op);
+    aws_mem_release(handler->alloc, input_data);
+    aws_byte_buf_clean_up(&output_buf);
+
+    if (success) {
+        s_drive_negotiation(handler);
+    }
+}
+
+static int s_s2n_pkcs11_async_pkey_callback(struct s2n_connection *conn, struct s2n_async_pkey_op *op) {
+    struct s2n_handler *s2n_handler = s2n_connection_get_ctx(conn);
+    struct aws_channel_handler *handler = &s2n_handler->handler;
+
+    AWS_ASSERT(conn == s2n_handler->connection);
+    (void)conn;
+
+    /* Schedule a task to do the work.
+     * s2n can't deal with the async private key operation completing synchronously, so we can't just do it now */
+    AWS_LOGF_TRACE(AWS_LS_IO_TLS, "id=%p: async pkey callback received, scheduling PKCS#11 task", (void *)handler);
+
+    aws_channel_task_init(&s2n_handler->async_pkey_task, s_s2n_pkcs11_async_pkey_task, op, "s2n_pkcs11_async_pkey_op");
+    aws_channel_schedule_task_now(s2n_handler->slot->channel, &s2n_handler->async_pkey_task);
+
+    return S2N_SUCCESS;
+}
+
 static int s_s2n_do_delayed_shutdown(
     struct aws_channel_handler *handler,
     struct aws_channel_slot *slot,
@@ -578,6 +865,11 @@ static int s_s2n_handler_shutdown(
         AWS_LOGF_DEBUG(
             AWS_LS_IO_TLS, "id=%p: Shutting down read direction with error code %d", (void *)handler, error_code);
 
+        /* If negotiation hasn't succeeded yet, it's certainly not going to succeed now */
+        if (s2n_handler->state == NEGOTIATION_ONGOING) {
+            s2n_handler->state = NEGOTIATION_FAILED;
+        }
+
         while (!aws_linked_list_empty(&s2n_handler->input_queue)) {
             struct aws_linked_list_node *node = aws_linked_list_pop_front(&s2n_handler->input_queue);
             struct aws_io_message *message = AWS_CONTAINER_OF(node, struct aws_io_message, queueing_handle);
@@ -626,7 +918,7 @@ static int s_s2n_handler_increment_read_window(
         aws_channel_slot_increment_read_window(slot, window_update_size);
     }
 
-    if (s2n_handler->negotiation_finished && !s2n_handler->sequential_tasks.node.next) {
+    if (s2n_handler->state == NEGOTIATION_SUCCEEDED && !s2n_handler->sequential_tasks.node.next) {
         /* TLS requires full records before it can decrypt anything. As a result we need to check everything we've
          * buffered instead of just waiting on a read from the socket, or we'll hit a deadlock.
          *
@@ -775,23 +1067,22 @@ static struct aws_channel_handler *s_new_tls_handler(
 
     AWS_ASSERT(options->ctx);
     struct s2n_handler *s2n_handler = aws_mem_calloc(allocator, 1, sizeof(struct s2n_handler));
-    if (!s2n_handler) {
-        return NULL;
-    }
-
-    struct s2n_ctx *s2n_ctx = (struct s2n_ctx *)options->ctx->impl;
-    s2n_handler->connection = s2n_connection_new(mode);
-
-    if (!s2n_handler->connection) {
-        goto cleanup_s2n_handler;
-    }
-
-    aws_tls_channel_handler_shared_init(&s2n_handler->shared_state, &s2n_handler->handler, options);
-
     s2n_handler->handler.impl = s2n_handler;
     s2n_handler->handler.alloc = allocator;
     s2n_handler->handler.vtable = &s_handler_vtable;
     s2n_handler->handler.slot = slot;
+
+    aws_tls_ctx_acquire(options->ctx);
+    s2n_handler->s2n_ctx = options->ctx->impl;
+
+    s2n_handler->connection = s2n_connection_new(mode);
+
+    if (!s2n_handler->connection) {
+        goto cleanup_conn;
+    }
+
+    aws_tls_channel_handler_shared_init(&s2n_handler->shared_state, &s2n_handler->handler, options);
+
     s2n_handler->user_data = options->user_data;
     s2n_handler->on_data_read = options->on_data_read;
     s2n_handler->on_error = options->on_error;
@@ -813,12 +1104,13 @@ static struct aws_channel_handler *s_new_tls_handler(
         }
     }
 
-    s2n_handler->negotiation_finished = false;
+    s2n_handler->state = NEGOTIATION_ONGOING;
 
     s2n_connection_set_recv_cb(s2n_handler->connection, s_s2n_handler_recv);
     s2n_connection_set_recv_ctx(s2n_handler->connection, s2n_handler);
     s2n_connection_set_send_cb(s2n_handler->connection, s_s2n_handler_send);
     s2n_connection_set_send_ctx(s2n_handler->connection, s2n_handler);
+    s2n_connection_set_ctx(s2n_handler->connection, s2n_handler);
     s2n_connection_set_blinding(s2n_handler->connection, S2N_SELF_SERVICE_BLINDING);
 
     if (options->alpn_list) {
@@ -849,7 +1141,7 @@ static struct aws_channel_handler *s_new_tls_handler(
         }
     }
 
-    if (s2n_connection_set_config(s2n_handler->connection, s2n_ctx->s2n_config)) {
+    if (s2n_connection_set_config(s2n_handler->connection, s2n_handler->s2n_ctx->s2n_config)) {
         AWS_LOGF_WARN(
             AWS_LS_IO_TLS,
             "id=%p: configuration error %s (%s)",
@@ -873,10 +1165,7 @@ static struct aws_channel_handler *s_new_tls_handler(
     return &s2n_handler->handler;
 
 cleanup_conn:
-    s2n_connection_free(s2n_handler->connection);
-
-cleanup_s2n_handler:
-    aws_mem_release(allocator, s2n_handler);
+    s_s2n_handler_destroy(&s2n_handler->handler);
 
     return NULL;
 }
@@ -899,7 +1188,17 @@ struct aws_channel_handler *aws_tls_server_handler_new(
 
 static void s_s2n_ctx_destroy(struct s2n_ctx *s2n_ctx) {
     if (s2n_ctx != NULL) {
+        if (s2n_ctx->pkcs11.session_handle != 0) {
+            aws_pkcs11_lib_close_session(s2n_ctx->pkcs11.lib, s2n_ctx->pkcs11.session_handle);
+        }
+        aws_mutex_clean_up(&s2n_ctx->pkcs11.session_lock);
+        aws_pkcs11_lib_release(s2n_ctx->pkcs11.lib);
         s2n_config_free(s2n_ctx->s2n_config);
+
+        if (s2n_ctx->custom_cert_chain_and_key) {
+            s2n_cert_chain_and_key_free(s2n_ctx->custom_cert_chain_and_key);
+        }
+
         aws_mem_release(s2n_ctx->ctx.alloc, s2n_ctx);
     }
 }
@@ -924,6 +1223,50 @@ static int s2n_monotonic_clock_time_nanoseconds(void *context, uint64_t *time_in
     return 0;
 }
 
+static int s_tls_ctx_pkcs11_setup(struct s2n_ctx *s2n_ctx, const struct aws_tls_ctx_options *options) {
+    /* PKCS#11 options were already sanitized (ie: check for required args) in tls_channel_handler.c */
+
+    /* anything initialized in this function is cleaned up during s_s2n_ctx_destroy()
+     * so don't worry about cleaning up unless it's some tmp heap allocation */
+
+    s2n_ctx->pkcs11.lib = aws_pkcs11_lib_acquire(options->pkcs11.lib); /* cannot fail */
+    aws_mutex_init(&s2n_ctx->pkcs11.session_lock);
+
+    CK_SLOT_ID slot_id = 0;
+    if (aws_pkcs11_lib_find_slot_with_token(
+            s2n_ctx->pkcs11.lib,
+            options->pkcs11.has_slot_id ? &options->pkcs11.slot_id : NULL,
+            options->pkcs11.token_label,
+            &slot_id /*out*/)) {
+        return AWS_OP_ERR;
+    }
+
+    if (aws_pkcs11_lib_open_session(s2n_ctx->pkcs11.lib, slot_id, &s2n_ctx->pkcs11.session_handle)) {
+        return AWS_OP_ERR;
+    }
+
+    if (aws_pkcs11_lib_login_user(s2n_ctx->pkcs11.lib, s2n_ctx->pkcs11.session_handle, options->pkcs11.user_pin)) {
+        return AWS_OP_ERR;
+    }
+
+    if (aws_pkcs11_lib_find_private_key(
+            s2n_ctx->pkcs11.lib,
+            s2n_ctx->pkcs11.session_handle,
+            options->pkcs11.private_key_object_label,
+            &s2n_ctx->pkcs11.private_key_handle /*out*/,
+            &s2n_ctx->pkcs11.private_key_type /*out*/)) {
+        return AWS_OP_ERR;
+    }
+
+    return AWS_OP_SUCCESS;
+}
+
+static void s_log_and_raise_s2n_errno(const char *msg) {
+    AWS_LOGF_ERROR(
+        AWS_LS_IO_TLS, "%s: %s (%s)", msg, s2n_strerror(s2n_errno, "EN"), s2n_strerror_debug(s2n_errno, "EN"));
+    aws_raise_error(AWS_IO_TLS_CTX_ERROR);
+}
+
 static struct aws_tls_ctx *s_tls_ctx_new(
     struct aws_allocator *alloc,
     const struct aws_tls_ctx_options *options,
@@ -943,41 +1286,69 @@ static struct aws_tls_ctx *s_tls_ctx_new(
     s2n_ctx->ctx.alloc = alloc;
     s2n_ctx->ctx.impl = s2n_ctx;
     aws_ref_count_init(&s2n_ctx->ctx.ref_count, s2n_ctx, (aws_simple_completion_callback *)s_s2n_ctx_destroy);
-    s2n_ctx->s2n_config = s2n_config_new();
 
+    s2n_ctx->s2n_config = s2n_config_new();
     if (!s2n_ctx->s2n_config) {
-        goto cleanup_s2n_ctx;
+        s_log_and_raise_s2n_errno("ctx: creation failed");
+        goto cleanup_s2n_config;
     }
 
     int set_clock_result = s2n_config_set_wall_clock(s2n_ctx->s2n_config, s2n_wall_clock_time_nanoseconds, NULL);
     if (set_clock_result != S2N_ERR_T_OK) {
+        s_log_and_raise_s2n_errno("ctx: failed to set wall clock");
         goto cleanup_s2n_config;
     }
 
     set_clock_result = s2n_config_set_monotonic_clock(s2n_ctx->s2n_config, s2n_monotonic_clock_time_nanoseconds, NULL);
     if (set_clock_result != S2N_ERR_T_OK) {
+        s_log_and_raise_s2n_errno("ctx: failed to set monotonic clock");
         goto cleanup_s2n_config;
     }
 
-    switch (options->minimum_tls_version) {
-        case AWS_IO_SSLv3:
-            s2n_config_set_cipher_preferences(s2n_ctx->s2n_config, "AWS-CRT-SDK-SSLv3.0");
-            break;
-        case AWS_IO_TLSv1:
-            s2n_config_set_cipher_preferences(s2n_ctx->s2n_config, "AWS-CRT-SDK-TLSv1.0");
-            break;
-        case AWS_IO_TLSv1_1:
-            s2n_config_set_cipher_preferences(s2n_ctx->s2n_config, "AWS-CRT-SDK-TLSv1.1");
-            break;
-        case AWS_IO_TLSv1_2:
-            s2n_config_set_cipher_preferences(s2n_ctx->s2n_config, "AWS-CRT-SDK-TLSv1.2");
-            break;
-        case AWS_IO_TLSv1_3:
-            s2n_config_set_cipher_preferences(s2n_ctx->s2n_config, "AWS-CRT-SDK-TLSv1.3");
-            break;
-        case AWS_IO_TLS_VER_SYS_DEFAULTS:
-        default:
-            s2n_config_set_cipher_preferences(s2n_ctx->s2n_config, "AWS-CRT-SDK-TLSv1.0");
+    if (options->pkcs11.lib != NULL) {
+        /* PKCS#11 integration hasn't been tested with TLS 1.3, so don't use cipher preferences that allow 1.3 */
+        switch (options->minimum_tls_version) {
+            case AWS_IO_SSLv3:
+                s2n_config_set_cipher_preferences(s2n_ctx->s2n_config, "CloudFront-SSL-v-3");
+                break;
+            case AWS_IO_TLSv1:
+                s2n_config_set_cipher_preferences(s2n_ctx->s2n_config, "CloudFront-TLS-1-0-2014");
+                break;
+            case AWS_IO_TLSv1_1:
+                s2n_config_set_cipher_preferences(s2n_ctx->s2n_config, "ELBSecurityPolicy-TLS-1-1-2017-01");
+                break;
+            case AWS_IO_TLSv1_2:
+                s2n_config_set_cipher_preferences(s2n_ctx->s2n_config, "ELBSecurityPolicy-TLS-1-2-Ext-2018-06");
+                break;
+            case AWS_IO_TLSv1_3:
+                AWS_LOGF_ERROR(AWS_LS_IO_TLS, "TLS 1.3 with PKCS#11 is not supported yet.");
+                aws_raise_error(AWS_IO_TLS_VERSION_UNSUPPORTED);
+                goto cleanup_s2n_config;
+            case AWS_IO_TLS_VER_SYS_DEFAULTS:
+            default:
+                s2n_config_set_cipher_preferences(s2n_ctx->s2n_config, "ELBSecurityPolicy-TLS-1-1-2017-01");
+        }
+    } else {
+        switch (options->minimum_tls_version) {
+            case AWS_IO_SSLv3:
+                s2n_config_set_cipher_preferences(s2n_ctx->s2n_config, "AWS-CRT-SDK-SSLv3.0");
+                break;
+            case AWS_IO_TLSv1:
+                s2n_config_set_cipher_preferences(s2n_ctx->s2n_config, "AWS-CRT-SDK-TLSv1.0");
+                break;
+            case AWS_IO_TLSv1_1:
+                s2n_config_set_cipher_preferences(s2n_ctx->s2n_config, "AWS-CRT-SDK-TLSv1.1");
+                break;
+            case AWS_IO_TLSv1_2:
+                s2n_config_set_cipher_preferences(s2n_ctx->s2n_config, "AWS-CRT-SDK-TLSv1.2");
+                break;
+            case AWS_IO_TLSv1_3:
+                s2n_config_set_cipher_preferences(s2n_ctx->s2n_config, "AWS-CRT-SDK-TLSv1.3");
+                break;
+            case AWS_IO_TLS_VER_SYS_DEFAULTS:
+            default:
+                s2n_config_set_cipher_preferences(s2n_ctx->s2n_config, "AWS-CRT-SDK-TLSv1.0");
+        }
     }
 
     switch (options->cipher_pref) {
@@ -1038,37 +1409,56 @@ static struct aws_tls_ctx *s_tls_ctx_new(
         }
 
         if (err_code != S2N_ERR_T_OK) {
-            AWS_LOGF_ERROR(
-                AWS_LS_IO_TLS,
-                "ctx: configuration error %s (%s)",
-                s2n_strerror(s2n_errno, "EN"),
-                s2n_strerror_debug(s2n_errno, "EN"));
-            aws_raise_error(AWS_IO_TLS_CTX_ERROR);
+            s_log_and_raise_s2n_errno("ctx: Failed to add certificate and private key");
             goto cleanup_s2n_config;
         }
+    } else if (options->pkcs11.lib != NULL) {
+        AWS_LOGF_DEBUG(AWS_LS_IO_TLS, "ctx: PKCS#11 has been set, setting it up now.");
+        if (s_tls_ctx_pkcs11_setup(s2n_ctx, options)) {
+            goto cleanup_s2n_config;
+        }
+
+        /* set callback so that we can do private key operations through PKCS#11 */
+        if (s2n_config_set_async_pkey_callback(s2n_ctx->s2n_config, s_s2n_pkcs11_async_pkey_callback)) {
+            s_log_and_raise_s2n_errno("ctx: failed to set private key callback");
+            goto cleanup_s2n_config;
+        }
+
+        /* set certificate.
+         * we need to create a custom s2n_cert_chain_and_key that knows the cert but not the key */
+        s2n_ctx->custom_cert_chain_and_key = s2n_cert_chain_and_key_new();
+        if (!s2n_ctx->custom_cert_chain_and_key) {
+            s_log_and_raise_s2n_errno("ctx: creation failed");
+            goto cleanup_s2n_config;
+        }
+
+        if (s2n_cert_chain_and_key_load_public_pem_bytes(
+                s2n_ctx->custom_cert_chain_and_key, options->certificate.buffer, options->certificate.len)) {
+            s_log_and_raise_s2n_errno("ctx: failed to load certificate");
+            goto cleanup_s2n_config;
+        }
+
+        if (s2n_config_add_cert_chain_and_key_to_store(s2n_ctx->s2n_config, s2n_ctx->custom_cert_chain_and_key)) {
+            s_log_and_raise_s2n_errno("ctx: failed to add certificate to store");
+            goto cleanup_s2n_config;
+        }
+
+        if (mode == S2N_CLIENT) {
+            s2n_config_set_client_auth_type(s2n_ctx->s2n_config, S2N_CERT_AUTH_REQUIRED);
+        }
     }
 
     if (options->verify_peer) {
         if (s2n_config_set_check_stapled_ocsp_response(s2n_ctx->s2n_config, 1) == S2N_SUCCESS) {
             if (s2n_config_set_status_request_type(s2n_ctx->s2n_config, S2N_STATUS_REQUEST_OCSP) != S2N_SUCCESS) {
-                AWS_LOGF_ERROR(
-                    AWS_LS_IO_TLS,
-                    "ctx: ocsp status request cannot be set: %s (%s)",
-                    s2n_strerror(s2n_errno, "EN"),
-                    s2n_strerror_debug(s2n_errno, "EN"));
-                aws_raise_error(AWS_IO_TLS_CTX_ERROR);
+                s_log_and_raise_s2n_errno("ctx: ocsp status request cannot be set");
                 goto cleanup_s2n_config;
             }
         } else {
             if (s2n_error_get_type(s2n_errno) == S2N_ERR_T_USAGE) {
                 AWS_LOGF_INFO(AWS_LS_IO_TLS, "ctx: cannot enable ocsp stapling: %s", s2n_strerror(s2n_errno, "EN"));
             } else {
-                AWS_LOGF_ERROR(
-                    AWS_LS_IO_TLS,
-                    "ctx: cannot enable ocsp stapling: %s (%s)",
-                    s2n_strerror(s2n_errno, "EN"),
-                    s2n_strerror_debug(s2n_errno, "EN"));
-                aws_raise_error(AWS_IO_TLS_CTX_ERROR);
+                s_log_and_raise_s2n_errno("ctx: cannot enable ocsp stapling");
                 goto cleanup_s2n_config;
             }
         }
@@ -1077,26 +1467,15 @@ static struct aws_tls_ctx *s_tls_ctx_new(
             /* The user called an override_default_trust_store() function.
              * Begin by wiping anything that s2n loaded by default */
             if (s2n_config_wipe_trust_store(s2n_ctx->s2n_config)) {
-                AWS_LOGF_ERROR(
-                    AWS_LS_IO_TLS,
-                    "ctx: configuration error %s (%s)",
-                    s2n_strerror(s2n_errno, "EN"),
-                    s2n_strerror_debug(s2n_errno, "EN"));
-                AWS_LOGF_ERROR(AWS_LS_IO_TLS, "Failed to wipe default trust store\n");
-                aws_raise_error(AWS_IO_TLS_CTX_ERROR);
+                s_log_and_raise_s2n_errno("ctx: failed to wipe default trust store");
                 goto cleanup_s2n_config;
             }
 
             if (options->ca_path) {
                 if (s2n_config_set_verification_ca_location(
                         s2n_ctx->s2n_config, NULL, aws_string_c_str(options->ca_path))) {
-                    AWS_LOGF_ERROR(
-                        AWS_LS_IO_TLS,
-                        "ctx: configuration error %s (%s)",
-                        s2n_strerror(s2n_errno, "EN"),
-                        s2n_strerror_debug(s2n_errno, "EN"));
+                    s_log_and_raise_s2n_errno("ctx: configuration error");
                     AWS_LOGF_ERROR(AWS_LS_IO_TLS, "Failed to set ca_path %s\n", aws_string_c_str(options->ca_path));
-                    aws_raise_error(AWS_IO_TLS_CTX_ERROR);
                     goto cleanup_s2n_config;
                 }
             }
@@ -1109,13 +1488,8 @@ static struct aws_tls_ctx *s_tls_ctx_new(
                 aws_string_destroy(ca_file_string);
 
                 if (set_ca_result) {
-                    AWS_LOGF_ERROR(
-                        AWS_LS_IO_TLS,
-                        "ctx: configuration error %s (%s)",
-                        s2n_strerror(s2n_errno, "EN"),
-                        s2n_strerror_debug(s2n_errno, "EN"));
+                    s_log_and_raise_s2n_errno("ctx: configuration error");
                     AWS_LOGF_ERROR(AWS_LS_IO_TLS, "Failed to set ca_file %s\n", (const char *)options->ca_file.buffer);
-                    aws_raise_error(AWS_IO_TLS_CTX_ERROR);
                     goto cleanup_s2n_config;
                 }
             }
@@ -1129,25 +1503,15 @@ static struct aws_tls_ctx *s_tls_ctx_new(
              * to multiple flavors of Linux). Therefore, load the locations that
              * were found at library startup. */
             if (s2n_config_set_verification_ca_location(s2n_ctx->s2n_config, s_default_ca_file, s_default_ca_dir)) {
-                AWS_LOGF_ERROR(
-                    AWS_LS_IO_TLS,
-                    "ctx: configuration error %s (%s)",
-                    s2n_strerror(s2n_errno, "EN"),
-                    s2n_strerror_debug(s2n_errno, "EN"));
+                s_log_and_raise_s2n_errno("ctx: configuration error");
                 AWS_LOGF_ERROR(
                     AWS_LS_IO_TLS, "Failed to set ca_path: %s and ca_file %s\n", s_default_ca_dir, s_default_ca_file);
-                aws_raise_error(AWS_IO_TLS_CTX_ERROR);
                 goto cleanup_s2n_config;
             }
         }
 
         if (mode == S2N_SERVER && s2n_config_set_client_auth_type(s2n_ctx->s2n_config, S2N_CERT_AUTH_REQUIRED)) {
-            AWS_LOGF_ERROR(
-                AWS_LS_IO_TLS,
-                "ctx: configuration error %s (%s)",
-                s2n_strerror(s2n_errno, "EN"),
-                s2n_strerror_debug(s2n_errno, "EN"));
-            aws_raise_error(AWS_IO_TLS_CTX_ERROR);
+            s_log_and_raise_s2n_errno("ctx: failed to set client auth type");
             goto cleanup_s2n_config;
         }
     } else if (mode != S2N_SERVER) {
@@ -1156,7 +1520,7 @@ static struct aws_tls_ctx *s_tls_ctx_new(
             "ctx: X.509 validation has been disabled. "
             "If this is not running in a test environment, this is likely a security vulnerability.");
         if (s2n_config_disable_x509_verification(s2n_ctx->s2n_config)) {
-            aws_raise_error(AWS_IO_TLS_CTX_ERROR);
+            s_log_and_raise_s2n_errno("ctx: failed to disable x509 verification");
             goto cleanup_s2n_config;
         }
     }
@@ -1167,7 +1531,7 @@ static struct aws_tls_ctx *s_tls_ctx_new(
         AWS_ZERO_ARRAY(protocols_cpy);
         size_t protocols_size = 4;
         if (s_parse_protocol_preferences(options->alpn_list, protocols_cpy, &protocols_size)) {
-            aws_raise_error(AWS_IO_TLS_CTX_ERROR);
+            s_log_and_raise_s2n_errno("ctx: Failed to parse ALPN list");
             goto cleanup_s2n_config;
         }
 
@@ -1178,7 +1542,7 @@ static struct aws_tls_ctx *s_tls_ctx_new(
         }
 
         if (s2n_config_set_protocol_preferences(s2n_ctx->s2n_config, protocols, (int)protocols_size)) {
-            aws_raise_error(AWS_IO_TLS_CTX_ERROR);
+            s_log_and_raise_s2n_errno("ctx: Failed to set protocol preferences");
             goto cleanup_s2n_config;
         }
     }
@@ -1196,10 +1560,7 @@ static struct aws_tls_ctx *s_tls_ctx_new(
     return &s2n_ctx->ctx;
 
 cleanup_s2n_config:
-    s2n_config_free(s2n_ctx->s2n_config);
-
-cleanup_s2n_ctx:
-    aws_mem_release(alloc, s2n_ctx);
+    s_s2n_ctx_destroy(s2n_ctx);
 
     return NULL;
 }
