@@ -234,7 +234,7 @@ DEFINE_METHOD_FUNCTION(struct built_in_curves, OPENSSL_built_in_curves) {
   out->curves[1].comment = "NIST P-384";
   out->curves[1].param_len = 48;
   out->curves[1].params = kP384Params;
-  out->curves[1].method = EC_GFp_mont_method();
+  out->curves[1].method = EC_GFp_nistp384_method();
 
   // 1.2.840.10045.3.1.7
   static const uint8_t kOIDP256[] = {0x2a, 0x86, 0x48, 0xce,
@@ -1250,7 +1250,10 @@ int EC_METHOD_get_field_type(const EC_METHOD *meth) {
 
 void EC_GROUP_set_point_conversion_form(EC_GROUP *group,
                                         point_conversion_form_t form) {
-  if (form != POINT_CONVERSION_UNCOMPRESSED) {
+  /* NO-OP. However, abort if consumer attempts to set a representation that is
+   * not supported. */
+  if (form != POINT_CONVERSION_UNCOMPRESSED &&
+      form != POINT_CONVERSION_COMPRESSED) {
     abort();
   }
 }
