@@ -32,6 +32,7 @@
 #include <openssl/dh.h>
 #include <openssl/digest.h>
 #include <openssl/ec.h>
+#include <openssl/evp.h>
 #include <openssl/ec_key.h>
 #include <openssl/ecdh.h>
 #include <openssl/ecdsa.h>
@@ -54,10 +55,13 @@ namespace acvp {
 #if defined(OPENSSL_TRUSTY)
 #include <trusty_log.h>
 #define LOG_ERROR(...) TLOGE(__VA_ARGS__)
+#define TLOG_TAG "modulewrapper"
 #else
 #define LOG_ERROR(...) fprintf(stderr, __VA_ARGS__)
 #endif  // OPENSSL_TRUSTY
 
+#define AES_GCM_NONCE_LENGTH 12
+
 constexpr size_t kMaxArgLength = (1 << 20);
 
 RequestBuffer::~RequestBuffer() = default;
@@ -303,7 +307,8 @@ static bool GetConfig(const Span<const uint8_t> args[], ReplyCallback write_repl
         }],
         "tagLen": [32, 64, 96, 104, 112, 120, 128],
         "ivLen": [96],
-        "ivGen": "external"
+        "ivGen": ["external", "internal"],
+        "ivGenMode": "8.2.2"
       },
       {
         "algorithm": "ACVP-AES-GMAC",
@@ -502,6 +507,7 @@ static bool GetConfig(const Span<const uint8_t> args[], ReplyCallback write_repl
             "P-521"
           ],
           "hashAlg": [
+            "SHA-1",
             "SHA2-224",
             "SHA2-256",
             "SHA2-384",
@@ -837,7 +843,8 @@ static bool GetConfig(const Span<const uint8_t> args[], ReplyCallback write_repl
         "scheme": {
           "dhEphem": {
             "kasRole": [
-              "initiator"
+              "initiator",
+              "responder"
             ]
           }
         },
@@ -859,6 +866,30 @@ static bool Hash(const Span<const uint8_t> args[], ReplyCallback write_reply) {
   return write_reply({Span<const uint8_t>(digest)});
 }
 
+template <uint8_t *(*OneShotHash)(const uint8_t *, size_t, uint8_t *),
+          size_t DigestLength>
+static bool HashMCT(const Span<const uint8_t> args[],
+                    ReplyCallback write_reply) {
+  if (args[0].size() != DigestLength) {
+    return false;
+  }
+
+  uint8_t buf[DigestLength * 3];
+  memcpy(buf, args[0].data(), DigestLength);
+  memcpy(buf + DigestLength, args[0].data(), DigestLength);
+  memcpy(buf + 2 * DigestLength, args[0].data(), DigestLength);
+
+  for (size_t i = 0; i < 1000; i++) {
+    uint8_t digest[DigestLength];
+    OneShotHash(buf, sizeof(buf), digest);
+    memmove(buf, buf + DigestLength, DigestLength * 2);
+    memcpy(buf + DigestLength * 2, digest, DigestLength);
+  }
+
+  return write_reply(
+      {Span<const uint8_t>(buf + 2 * DigestLength, DigestLength)});
+}
+
 static uint32_t GetIterations(const Span<const uint8_t> iterations_bytes) {
   uint32_t iterations;
   if (iterations_bytes.size() != sizeof(iterations)) {
@@ -989,7 +1020,7 @@ static bool AES_CTR(const Span<const uint8_t> args[], ReplyCallback write_reply)
 }
 
 static bool AESGCMSetup(EVP_AEAD_CTX *ctx, Span<const uint8_t> tag_len_span,
-                        Span<const uint8_t> key) {
+                        Span<const uint8_t> key, Span<const uint8_t> nonce) {
   uint32_t tag_len_32;
   if (tag_len_span.size() != sizeof(tag_len_32)) {
     LOG_ERROR("Tag size value is %u bytes, not an uint32_t\n",
@@ -999,33 +1030,61 @@ static bool AESGCMSetup(EVP_AEAD_CTX *ctx, Span<const uint8_t> tag_len_span,
   memcpy(&tag_len_32, tag_len_span.data(), sizeof(tag_len_32));
 
   const EVP_AEAD *aead;
-  switch (key.size()) {
-    case 16:
-      aead = EVP_aead_aes_128_gcm();
-      break;
-    case 24:
-      aead = EVP_aead_aes_192_gcm();
-      break;
-    case 32:
-      aead = EVP_aead_aes_256_gcm();
-      break;
-    default:
-      LOG_ERROR("Bad AES-GCM key length %u\n", static_cast<unsigned>(key.size()));
+  if(nonce.empty()) {
+    // Internally generated IVs
+    switch (key.size()) {
+      case 16:
+        aead = EVP_aead_aes_128_gcm_randnonce();
+        break;
+      case 32:
+        aead = EVP_aead_aes_256_gcm_randnonce();
+        break;
+      default:
+        LOG_ERROR("Bad AES-GCM key length %u\n",
+                  static_cast<unsigned>(key.size()));
+        return false;
+    }
+    // The 12-byte nonce is appended to the tag and is generated internally for
+    // random nonce function. Thus, the "tag" must be extended by 12 bytes
+    // for the purpose of the API.
+    if (!EVP_AEAD_CTX_init(ctx, aead, key.data(), key.size(), tag_len_32 + AES_GCM_NONCE_LENGTH,
+                           nullptr)) {
+      LOG_ERROR("Failed to setup AES-GCM with tag length %u\n",
+                static_cast<unsigned>(tag_len_32));
       return false;
+    }
+  } else {
+    // External IVs
+    switch (key.size()) {
+      case 16:
+        aead = EVP_aead_aes_128_gcm_tls12();
+        break;
+      case 24:
+        aead = EVP_aead_aes_192_gcm();
+        break;
+      case 32:
+        aead = EVP_aead_aes_256_gcm_tls12();
+        break;
+      default:
+        LOG_ERROR("Bad AES-GCM key length %u\n",
+                  static_cast<unsigned>(key.size()));
+        return false;
+    }
+    if (!EVP_AEAD_CTX_init(ctx, aead, key.data(), key.size(), tag_len_32,
+                           nullptr)) {
+      LOG_ERROR("Failed to setup AES-GCM with tag length %u\n",
+                static_cast<unsigned>(tag_len_32));
+      return false;
+    }
   }
 
-  if (!EVP_AEAD_CTX_init(ctx, aead, key.data(), key.size(), tag_len_32,
-                         nullptr)) {
-    LOG_ERROR("Failed to setup AES-GCM with tag length %u\n",
-              static_cast<unsigned>(tag_len_32));
-    return false;
-  }
+
 
   return true;
 }
 
 static bool AESCCMSetup(EVP_AEAD_CTX *ctx, Span<const uint8_t> tag_len_span,
-                        Span<const uint8_t> key) {
+                        Span<const uint8_t> key, Span<const uint8_t> nonce) {
   uint32_t tag_len_32;
   if (tag_len_span.size() != sizeof(tag_len_32)) {
     LOG_ERROR("Tag size value is %u bytes, not an uint32_t\n",
@@ -1056,7 +1115,7 @@ static bool AESCCMSetup(EVP_AEAD_CTX *ctx, Span<const uint8_t> tag_len_span,
 }
 
 template <bool (*SetupFunc)(EVP_AEAD_CTX *ctx, Span<const uint8_t> tag_len_span,
-                            Span<const uint8_t> key)>
+                            Span<const uint8_t> key, Span<const uint8_t> nonce)>
 static bool AEADSeal(const Span<const uint8_t> args[], ReplyCallback write_reply) {
   Span<const uint8_t> tag_len_span = args[0];
   Span<const uint8_t> key = args[1];
@@ -1065,7 +1124,7 @@ static bool AEADSeal(const Span<const uint8_t> args[], ReplyCallback write_reply
   Span<const uint8_t> ad = args[4];
 
   bssl::ScopedEVP_AEAD_CTX ctx;
-  if (!SetupFunc(ctx.get(), tag_len_span, key)) {
+  if (!SetupFunc(ctx.get(), tag_len_span, key, nonce)) {
     return false;
   }
 
@@ -1075,10 +1134,20 @@ static bool AEADSeal(const Span<const uint8_t> args[], ReplyCallback write_reply
   std::vector<uint8_t> out(EVP_AEAD_MAX_OVERHEAD + plaintext.size());
 
   size_t out_len;
-  if (!EVP_AEAD_CTX_seal(ctx.get(), out.data(), &out_len, out.size(),
-                         nonce.data(), nonce.size(), plaintext.data(),
-                         plaintext.size(), ad.data(), ad.size())) {
-    return false;
+  if(nonce.empty()) {
+    // The nonce parameter when using Internal IV generation must be zero-length.
+    if (!EVP_AEAD_CTX_seal(ctx.get(), out.data(), &out_len, out.size(),
+                           nullptr, 0, plaintext.data(),
+                           plaintext.size(), ad.data(), ad.size())) {
+      return false;
+    }
+  } else {
+    // External IV AEAD sealing.
+    if (!EVP_AEAD_CTX_seal(ctx.get(), out.data(), &out_len, out.size(),
+                           nonce.data(), nonce.size(), plaintext.data(),
+                           plaintext.size(), ad.data(), ad.size())) {
+      return false;
+    }
   }
 
   out.resize(out_len);
@@ -1086,7 +1155,7 @@ static bool AEADSeal(const Span<const uint8_t> args[], ReplyCallback write_reply
 }
 
 template <bool (*SetupFunc)(EVP_AEAD_CTX *ctx, Span<const uint8_t> tag_len_span,
-                            Span<const uint8_t> key)>
+                            Span<const uint8_t> key, Span<const uint8_t> nonce)>
 static bool AEADOpen(const Span<const uint8_t> args[], ReplyCallback write_reply) {
   Span<const uint8_t> tag_len_span = args[0];
   Span<const uint8_t> key = args[1];
@@ -1095,7 +1164,7 @@ static bool AEADOpen(const Span<const uint8_t> args[], ReplyCallback write_reply
   Span<const uint8_t> ad = args[4];
 
   bssl::ScopedEVP_AEAD_CTX ctx;
-  if (!SetupFunc(ctx.get(), tag_len_span, key)) {
+  if (!SetupFunc(ctx.get(), tag_len_span, key, nonce)) {
     return false;
   }
 
@@ -1481,7 +1550,9 @@ static bool ECDSAKeyVer(const Span<const uint8_t> args[], ReplyCallback write_re
 }
 
 static const EVP_MD *HashFromName(Span<const uint8_t> name) {
-  if (StringEq(name, "SHA2-224")) {
+  if (StringEq(name, "SHA-1")) {
+    return EVP_sha1();
+  } else if (StringEq(name, "SHA2-224")) {
     return EVP_sha224();
   } else if (StringEq(name, "SHA2-256")) {
     return EVP_sha256();
@@ -1498,16 +1569,27 @@ static bool ECDSASigGen(const Span<const uint8_t> args[], ReplyCallback write_re
   bssl::UniquePtr<EC_KEY> key = ECKeyFromName(args[0]);
   bssl::UniquePtr<BIGNUM> d = BytesToBIGNUM(args[1]);
   const EVP_MD *hash = HashFromName(args[2]);
-  uint8_t digest[EVP_MAX_MD_SIZE];
-  unsigned digest_len;
-  if (!key || !hash ||
-      !EVP_Digest(args[3].data(), args[3].size(), digest, &digest_len, hash,
-                  /*impl=*/nullptr) ||
-      !EC_KEY_set_private_key(key.get(), d.get())) {
+  auto msg = args[3];
+  if (!key || !hash || !EC_KEY_set_private_key(key.get(), d.get())) {
     return false;
   }
-
-  bssl::UniquePtr<ECDSA_SIG> sig(ECDSA_do_sign(digest, digest_len, key.get()));
+  bssl::ScopedEVP_MD_CTX ctx;
+  EVP_PKEY_CTX *pctx;
+  bssl::UniquePtr<EVP_PKEY> evp_pkey(EVP_PKEY_new());
+  if (!evp_pkey || !EVP_PKEY_set1_EC_KEY(evp_pkey.get(), key.get())) {
+    return false;
+  }
+  std::vector<uint8_t> sig_der;
+  size_t len;
+  if (!EVP_DigestSignInit(ctx.get(), &pctx, hash, nullptr, evp_pkey.get()) ||
+      !EVP_DigestSign(ctx.get(), nullptr, &len, msg.data(), msg.size())) {
+    return false;
+  }
+  sig_der.resize(len);
+  if (!EVP_DigestSign(ctx.get(), sig_der.data(), &len, msg.data(), msg.size())) {
+    return false;
+  }
+  bssl::UniquePtr<ECDSA_SIG> sig(ECDSA_SIG_from_bytes(sig_der.data(), len));
   if (!sig) {
     return false;
   }
@@ -1530,27 +1612,35 @@ static bool ECDSASigVer(const Span<const uint8_t> args[], ReplyCallback write_re
   ECDSA_SIG sig;
   sig.r = r.get();
   sig.s = s.get();
-
-  uint8_t digest[EVP_MAX_MD_SIZE];
-  unsigned digest_len;
-  if (!key || !hash ||
-      !EVP_Digest(msg.data(), msg.size(), digest, &digest_len, hash,
-                  /*impl=*/nullptr)) {
+  uint8_t *der;
+  size_t der_len;
+  if (!key || !hash || !ECDSA_SIG_to_bytes(&der, &der_len, &sig)) {
     return false;
   }
-
+  // Let |delete_der| manage the release of |der|.
+  bssl::UniquePtr<uint8_t> delete_der(der);
   bssl::UniquePtr<EC_POINT> point(EC_POINT_new(EC_KEY_get0_group(key.get())));
-  uint8_t reply[1];
   if (!EC_POINT_set_affine_coordinates_GFp(EC_KEY_get0_group(key.get()),
                                            point.get(), x.get(), y.get(),
                                            /*ctx=*/nullptr) ||
       !EC_KEY_set_public_key(key.get(), point.get()) ||
-      !EC_KEY_check_fips(key.get()) ||
-      !ECDSA_do_verify(digest, digest_len, &sig, key.get())) {
+      !EC_KEY_check_fips(key.get())) {
+    return false;
+  }
+  bssl::ScopedEVP_MD_CTX ctx;
+  EVP_PKEY_CTX *pctx;
+  bssl::UniquePtr<EVP_PKEY> evp_pkey(EVP_PKEY_new());
+  if (!evp_pkey || !EVP_PKEY_set1_EC_KEY(evp_pkey.get(), key.get())) {
+    return false;
+  }
+  uint8_t reply[1];
+  if (!EVP_DigestVerifyInit(ctx.get(), &pctx, hash, nullptr, evp_pkey.get()) ||
+      !EVP_DigestVerify(ctx.get(), der, der_len, msg.data(), msg.size())) {
     reply[0] = 0;
   } else {
     reply[0] = 1;
   }
+  ERR_clear_error();
 
   return write_reply({Span<const uint8_t>(reply)});
 }
@@ -1588,26 +1678,34 @@ static bool CMAC_AESVerify(const Span<const uint8_t> args[], ReplyCallback write
   return write_reply({Span<const uint8_t>(&ok, sizeof(ok))});
 }
 
-static std::map<unsigned, bssl::UniquePtr<RSA>>& CachedRSAKeys() {
-  static std::map<unsigned, bssl::UniquePtr<RSA>> keys;
+static std::map<unsigned, bssl::UniquePtr<EVP_PKEY>>& CachedRSAEVPKeys() {
+  static std::map<unsigned, bssl::UniquePtr<EVP_PKEY>> keys;
   return keys;
 }
 
-static RSA* GetRSAKey(unsigned bits) {
-  auto it = CachedRSAKeys().find(bits);
-  if (it != CachedRSAKeys().end()) {
+static EVP_PKEY* AddRSAKeyToCache(bssl::UniquePtr<RSA>& rsa, unsigned bits) {
+  bssl::UniquePtr<EVP_PKEY> evp_pkey(EVP_PKEY_new());
+  if (!evp_pkey || !EVP_PKEY_set1_RSA(evp_pkey.get(), rsa.get())) {
+    return nullptr;
+  }
+
+  EVP_PKEY *const ret = evp_pkey.get();
+  CachedRSAEVPKeys().emplace(static_cast<unsigned>(bits), std::move(evp_pkey));
+  return ret;
+}
+
+static EVP_PKEY* GetRSAKey(unsigned bits) {
+  auto it = CachedRSAEVPKeys().find(bits);
+  if (it != CachedRSAEVPKeys().end()) {
     return it->second.get();
   }
 
-  bssl::UniquePtr<RSA> key(RSA_new());
-  if (!RSA_generate_key_fips(key.get(), bits, nullptr)) {
+  bssl::UniquePtr<RSA> rsa(RSA_new());
+  if (!RSA_generate_key_fips(rsa.get(), bits, nullptr)) {
     abort();
   }
 
-  RSA *const ret = key.get();
-  CachedRSAKeys().emplace(static_cast<unsigned>(bits), std::move(key));
-
-  return ret;
+  return AddRSAKeyToCache(rsa, bits);
 }
 
 static bool RSAKeyGen(const Span<const uint8_t> args[], ReplyCallback write_reply) {
@@ -1617,22 +1715,24 @@ static bool RSAKeyGen(const Span<const uint8_t> args[], ReplyCallback write_repl
   }
   memcpy(&bits, args[0].data(), sizeof(bits));
 
-  bssl::UniquePtr<RSA> key(RSA_new());
-  if (!RSA_generate_key_fips(key.get(), bits, nullptr)) {
+  bssl::UniquePtr<RSA> rsa(RSA_new());
+  if (!RSA_generate_key_fips(rsa.get(), bits, nullptr)) {
     LOG_ERROR("RSA_generate_key_fips failed for modulus length %u.\n", bits);
     return false;
   }
 
   const BIGNUM *n, *e, *d, *p, *q;
-  RSA_get0_key(key.get(), &n, &e, &d);
-  RSA_get0_factors(key.get(), &p, &q);
+  RSA_get0_key(rsa.get(), &n, &e, &d);
+  RSA_get0_factors(rsa.get(), &p, &q);
 
   if (!write_reply({BIGNUMBytes(e), BIGNUMBytes(p), BIGNUMBytes(q),
                     BIGNUMBytes(n), BIGNUMBytes(d)})) {
     return false;
   }
 
-  CachedRSAKeys().emplace(static_cast<unsigned>(bits), std::move(key));
+  if (AddRSAKeyToCache(rsa, bits) == nullptr) {
+    return false;
+  }
   return true;
 }
 
@@ -1644,35 +1744,33 @@ static bool RSASigGen(const Span<const uint8_t> args[], ReplyCallback write_repl
   }
   memcpy(&bits, args[0].data(), sizeof(bits));
   const Span<const uint8_t> msg = args[1];
-
-  RSA *const key = GetRSAKey(bits);
+  EVP_PKEY *const evp_pkey = GetRSAKey(bits);
+  if (evp_pkey == nullptr) {
+    return false;
+  }
+  RSA *const rsa = EVP_PKEY_get0_RSA(evp_pkey);
+  if (rsa == nullptr) {
+    return false;
+  }
   const EVP_MD *const md = MDFunc();
-  uint8_t digest_buf[EVP_MAX_MD_SIZE];
-  unsigned digest_len;
-  if (!EVP_Digest(msg.data(), msg.size(), digest_buf, &digest_len, md, NULL)) {
+  std::vector<uint8_t> sig;
+  size_t sig_len;
+  bssl::ScopedEVP_MD_CTX ctx;
+  EVP_PKEY_CTX *pctx;
+  int padding = UsePSS ? RSA_PKCS1_PSS_PADDING : RSA_PKCS1_PADDING;
+  if (!EVP_DigestSignInit(ctx.get(), &pctx, md, nullptr, evp_pkey) ||
+      !EVP_PKEY_CTX_set_rsa_padding(pctx, padding) ||
+      (UsePSS && !EVP_PKEY_CTX_set_rsa_pss_saltlen(pctx, RSA_PSS_SALTLEN_DIGEST)) ||
+      !EVP_DigestSign(ctx.get(), nullptr, &sig_len, msg.data(), msg.size())) {
+    return false;
+  }
+  sig.resize(sig_len);
+  if (!EVP_DigestSign(ctx.get(), sig.data(), &sig_len, msg.data(), msg.size())) {
     return false;
   }
 
-  std::vector<uint8_t> sig(RSA_size(key));
-  size_t sig_len;
-  if (UsePSS) {
-    if (!RSA_sign_pss_mgf1(key, &sig_len, sig.data(), sig.size(), digest_buf,
-                           digest_len, md, md, -1)) {
-      return false;
-    }
-  } else {
-    unsigned sig_len_u;
-    if (!RSA_sign(EVP_MD_type(md), digest_buf, digest_len, sig.data(),
-                  &sig_len_u, key)) {
-      return false;
-    }
-    sig_len = sig_len_u;
-  }
-
-  sig.resize(sig_len);
-
   return write_reply(
-      {BIGNUMBytes(RSA_get0_n(key)), BIGNUMBytes(RSA_get0_e(key)), sig});
+      {BIGNUMBytes(RSA_get0_n(rsa)), BIGNUMBytes(RSA_get0_e(rsa)), sig});
 }
 
 template <const EVP_MD *(MDFunc)(), bool UsePSS>
@@ -1692,19 +1790,21 @@ static bool RSASigVer(const Span<const uint8_t> args[], ReplyCallback write_repl
   }
 
   const EVP_MD *const md = MDFunc();
-  uint8_t digest_buf[EVP_MAX_MD_SIZE];
-  unsigned digest_len;
-  if (!EVP_Digest(msg.data(), msg.size(), digest_buf, &digest_len, md, NULL)) {
+  bssl::ScopedEVP_MD_CTX ctx;
+  EVP_PKEY_CTX *pctx;
+  bssl::UniquePtr<EVP_PKEY> evp_pkey(EVP_PKEY_new());
+  if (!evp_pkey || !EVP_PKEY_set1_RSA(evp_pkey.get(), key.get())) {
     return false;
   }
-
   uint8_t ok;
-  if (UsePSS) {
-    ok = RSA_verify_pss_mgf1(key.get(), digest_buf, digest_len, md, md, -1,
-                             sig.data(), sig.size());
+  int padding = UsePSS ? RSA_PKCS1_PSS_PADDING : RSA_PKCS1_PADDING;
+  if (!EVP_DigestVerifyInit(ctx.get(), &pctx, md, nullptr, evp_pkey.get()) ||
+      !EVP_PKEY_CTX_set_rsa_padding(pctx, padding) ||
+      (UsePSS && !EVP_PKEY_CTX_set_rsa_pss_saltlen(pctx, RSA_PSS_SALTLEN_DIGEST)) ||
+      !EVP_DigestVerify(ctx.get(), sig.data(), sig.size(), msg.data(), msg.size())) {
+    ok = 0;
   } else {
-    ok = RSA_verify(EVP_MD_type(md), digest_buf, digest_len, sig.data(),
-                    sig.size(), key.get());
+    ok = 1;
   }
   ERR_clear_error();
 
@@ -1858,6 +1958,12 @@ static constexpr struct {
     {"SHA2-384", 1, Hash<SHA384, SHA384_DIGEST_LENGTH>},
     {"SHA2-512", 1, Hash<SHA512, SHA512_DIGEST_LENGTH>},
     {"SHA2-512/256", 1, Hash<SHA512_256, SHA512_256_DIGEST_LENGTH>},
+    {"SHA-1/MCT", 1, HashMCT<SHA1, SHA_DIGEST_LENGTH>},
+    {"SHA2-224/MCT", 1, HashMCT<SHA224, SHA224_DIGEST_LENGTH>},
+    {"SHA2-256/MCT", 1, HashMCT<SHA256, SHA256_DIGEST_LENGTH>},
+    {"SHA2-384/MCT", 1, HashMCT<SHA384, SHA384_DIGEST_LENGTH>},
+    {"SHA2-512/MCT", 1, HashMCT<SHA512, SHA512_DIGEST_LENGTH>},
+    {"SHA2-512/256/MCT", 1, HashMCT<SHA512_256, SHA512_256_DIGEST_LENGTH>},
     {"AES/encrypt", 3, AES<AES_set_encrypt_key, AES_encrypt>},
     {"AES/decrypt", 3, AES<AES_set_decrypt_key, AES_decrypt>},
     {"AES-CBC/encrypt", 4, AES_CBC<AES_set_encrypt_key, AES_ENCRYPT>},
