@@ -286,12 +286,12 @@ static S2N_RESULT s2n_match_psk_identity(struct s2n_array *known_psks, const str
 }
 
 /**
- *= https://tools.ietf.org/html/rfc8446#section-4.2.10  
+ *= https://tools.ietf.org/rfc/rfc8446#section-4.2.10
  *# For PSKs provisioned via NewSessionTicket, a server MUST validate
- *#  that the ticket age for the selected PSK identity (computed by
- *#  subtracting ticket_age_add from PskIdentity.obfuscated_ticket_age
- *#  modulo 2^32) is within a small tolerance of the time since the ticket
- *#  was issued (see Section 8).
+ *# that the ticket age for the selected PSK identity (computed by
+ *# subtracting ticket_age_add from PskIdentity.obfuscated_ticket_age
+ *# modulo 2^32) is within a small tolerance of the time since the ticket
+ *# was issued (see Section 8).
  **/
 static S2N_RESULT s2n_validate_ticket_lifetime(struct s2n_connection *conn, uint32_t obfuscated_ticket_age, uint32_t ticket_age_add) 
 {
@@ -316,19 +316,21 @@ int s2n_offered_psk_list_choose_psk(struct s2n_offered_psk_list *psk_list, struc
     POSIX_ENSURE_REF(psk_list->conn);
 
     struct s2n_psk_parameters *psk_params = &psk_list->conn->psk_params;
+    struct s2n_stuffer ticket_stuffer = { 0 };
 
     if (!psk) {
         psk_params->chosen_psk = NULL;
         return S2N_SUCCESS;
     }
-    
+
     if (psk_params->type == S2N_PSK_TYPE_RESUMPTION && psk_list->conn->config->use_tickets) {
-        POSIX_GUARD(s2n_stuffer_init(&psk_list->conn->client_ticket_to_decrypt, &psk->identity));
-        POSIX_GUARD(s2n_stuffer_skip_write(&psk_list->conn->client_ticket_to_decrypt, psk->identity.size));
+        POSIX_GUARD(s2n_stuffer_init(&ticket_stuffer, &psk->identity));
+        POSIX_GUARD(s2n_stuffer_skip_write(&ticket_stuffer, psk->identity.size));
 
         /* s2n_decrypt_session_ticket appends a new PSK with the decrypted values. */
-        POSIX_GUARD(s2n_decrypt_session_ticket(psk_list->conn));
+        POSIX_GUARD(s2n_decrypt_session_ticket(psk_list->conn, &ticket_stuffer));
     }
+
     struct s2n_psk *chosen_psk = NULL;
     POSIX_GUARD_RESULT(s2n_match_psk_identity(&psk_params->psk_list, &psk->identity, &chosen_psk));
     POSIX_ENSURE_REF(chosen_psk);
