@@ -60,8 +60,8 @@ static int s2n_tls13_keys_init_with_ref(struct s2n_tls13_keys *handshake, s2n_hm
 
 int s2n_tls13_keys_from_conn(struct s2n_tls13_keys *keys, struct s2n_connection *conn)
 {
-    POSIX_GUARD(s2n_tls13_keys_init_with_ref(keys, conn->secure.cipher_suite->prf_alg, conn->secrets.rsa_premaster_secret, conn->secrets.master_secret));
-
+    POSIX_GUARD(s2n_tls13_keys_init_with_ref(keys, conn->secure.cipher_suite->prf_alg,
+            conn->secrets.tls13.extracted_secret, conn->secrets.tls13.derived_secret));
     return S2N_SUCCESS;
 }
 
@@ -341,12 +341,12 @@ static int s2n_tls13_handle_application_secret(struct s2n_connection *conn, s2n_
     struct s2n_session_key *session_key;
     s2n_secret_type_t secret_type;
     if (mode == S2N_CLIENT) {
-        app_secret_data = conn->secrets.client_app_secret;
+        app_secret_data = conn->secrets.tls13.client_app_secret;
         implicit_iv_data = conn->secure.client_implicit_iv;
         session_key = &conn->secure.client_key;
         secret_type = S2N_CLIENT_APPLICATION_TRAFFIC_SECRET;
     } else {
-        app_secret_data = conn->secrets.server_app_secret;
+        app_secret_data = conn->secrets.tls13.server_app_secret;
         implicit_iv_data = conn->secure.server_implicit_iv;
         session_key = &conn->secure.server_key;
         secret_type = S2N_SERVER_APPLICATION_TRAFFIC_SECRET;
@@ -409,7 +409,7 @@ static int s2n_tls13_handle_resumption_master_secret(struct s2n_connection *conn
     POSIX_GUARD(s2n_handshake_get_hash_state(conn, keys.hash_algorithm, &hash_state));
     
     struct s2n_blob resumption_master_secret = {0};
-    POSIX_GUARD(s2n_blob_init(&resumption_master_secret, conn->resumption_master_secret, keys.size));
+    POSIX_GUARD(s2n_blob_init(&resumption_master_secret, conn->secrets.tls13.resumption_master_secret, keys.size));
     POSIX_GUARD(s2n_tls13_derive_resumption_master_secret(&keys, &hash_state, &resumption_master_secret));
     return S2N_SUCCESS;
 }
@@ -522,11 +522,11 @@ int s2n_update_application_traffic_keys(struct s2n_connection *conn, s2n_mode mo
 
     if (mode == S2N_CLIENT) {
         old_key = &conn->secure.client_key;
-        POSIX_GUARD(s2n_blob_init(&old_app_secret, conn->secrets.client_app_secret, keys.size));
+        POSIX_GUARD(s2n_blob_init(&old_app_secret, conn->secrets.tls13.client_app_secret, keys.size));
         POSIX_GUARD(s2n_blob_init(&app_iv, conn->secure.client_implicit_iv, S2N_TLS13_FIXED_IV_LEN));
     } else {
         old_key = &conn->secure.server_key;
-        POSIX_GUARD(s2n_blob_init(&old_app_secret, conn->secrets.server_app_secret, keys.size));
+        POSIX_GUARD(s2n_blob_init(&old_app_secret, conn->secrets.tls13.server_app_secret, keys.size));
         POSIX_GUARD(s2n_blob_init(&app_iv, conn->secure.server_implicit_iv, S2N_TLS13_FIXED_IV_LEN));  
     }
 
